-------------------------------------------------------------------------------
-- Title      : firUnit
-- Project    : 
-------------------------------------------------------------------------------
-- File       : operativeUnit.vhd
-- Author     : Jean-Noel BAZIN  <jnbazin@pc-disi-026.enst-bretagne.fr>
-- Company    : 
-- Created    : 2018-04-11
-- Last update: 2018-04-11
-- Platform   : 
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: 8 bit FIR
-------------------------------------------------------------------------------
-- Copyright (c) 2018 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2018-04-11  1.0      jnbazin Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity firUnit is

  port (
    I_clock               : in  std_logic;  -- global clock
    I_reset               : in  std_logic;  -- asynchronous global reset
    I_inputSample         : in  std_logic_vector(11 downto 0);  -- 8 bit input sample
    I_inputSampleValid    : in  std_logic;
    O_filteredSample      : out std_logic_vector(11 downto 0);  -- filtered sample
    O_filteredSampleValid : out std_logic
    );

end entity firUnit;

architecture archi_firUnit of firUnit is

  component controlUnit is
    port (
    I_clock             : in  std_logic;  -- global clock
    I_reset               : in  std_logic;  -- asynchronous global reset
    I_inputSampleValid    : in  std_logic;  -- Control signal to load the input sample in the sample shift register and shift the register
    I_processingDoneA     : in  std_logic;
    I_processingDoneB     : in  std_logic;
    I_processingDoneBL: in std_logic;
    I_processingDoneClellan : in std_logic;
    O_loadShift           : out std_logic;  -- filtered sample
    O_initAddress         : out std_logic;  -- Control signal to initialize register read address
    O_incrAddress         : out std_logic;  -- Control signal to increment register read address
    O_initSum             : out std_logic;  -- Control signal to initialize the MAC register
    O_loadSum             : out std_logic;  -- Control signal to load the MAC register;
    O_loadOutB            : out std_logic;  -- Control signal to load 2A register
    O_loadOutA            : out std_logic;  -- Control signal to load Y register
    O_loadOutBL     : out  std_logic;
    O_loadOutClellan      : out  std_logic;
    O_sel                 : out std_logic_vector(1 downto 0); 
    O_FilteredSampleValid : out std_logic );
  end component controlUnit;

  component operativeUnit is
    port (
    I_clock          : in  std_logic;   -- global clock
    I_reset          : in  std_logic;   -- asynchronous global reset
    I_inputSample    : in  std_logic_vector(11 downto 0);  -- 8 bit input sample
    I_loadShift      : in  std_logic;  -- Control signal to load the input sample in the sample shift register and shift the register
    I_initAddress    : in  std_logic;  -- Control signal to initialize register read address
    I_incrAddress    : in  std_logic;  -- Control signal to increment register read address
    I_sel            : in  std_logic_vector(1 downto 0);  -- Control signal to control the type of process
    I_initSum        : in  std_logic;  -- Control signal to initialize the MAC register
    I_loadSum        : in  std_logic;  -- Control signal to load the MAC register;
    I_loadOutA       : in  std_logic;   -- Control signal to load Y register coefficients A
    I_loadOutB       : in  std_logic;
    I_loadOutBL : in  std_logic; 
    I_loadOutClellan  : in  std_logic;
    O_processingDoneA : out std_logic;   -- Indicate that processing is done
    O_processingDoneB : out std_logic;   -- Indicate that processing is done
    O_processingDoneBL: out std_logic;
    O_processingDoneClellan : out std_logic;
    O_total           : out std_logic_vector(11 downto 0));
    
  end component operativeUnit;

  signal SC_processingDoneA : std_logic;
  signal SC_processingDoneB : std_logic;
  signal SC_processingDoneBaseLine: std_logic;
  signal SC_processingDoneLissage : std_logic;
  signal SC_loadShift      : std_logic;
  signal SC_initAddress    : std_logic;
  signal SC_incrAddress    : std_logic;
  signal SC_initSum        : std_logic;
  signal SC_loadSum        : std_logic;
  signal SC_loadOutA       : std_logic;
  signal SC_loadOutB       : std_logic;
  signal SC_loadOutBaseLine: std_logic;
  signal SC_loadOutLissage : std_logic;
  signal SC_sel            : std_logic_vector(1 downto 0);
  
begin

  controlUnit_1 : entity work.controlUnit
    port map (
      I_clock               => I_clock,
      I_reset               => I_reset,
      I_inputSampleValid    => I_inputSampleValid,
      I_processingDoneA     => SC_processingDoneA,
      I_processingDoneB     => SC_processingDoneB,
      I_processingDoneBL  => SC_processingDoneBaseLine,
      I_processingDoneClellan   => SC_processingDoneLissage,
      O_loadShift           => SC_loadShift,
      O_initAddress         => SC_initAddress,
      O_incrAddress         => SC_incrAddress,
      O_initSum             => SC_initSum,
      O_loadSum             => SC_loadSum,
      O_loadOutA            => SC_loadOutA,
      O_loadOutB            => SC_loadOutB,
      O_loadOutBL     => SC_loadOutBaseLine,
      O_loadOutClellan      => SC_loadOutLissage,
      O_sel                 => SC_sel,
      O_FilteredSampleValid => O_FilteredSampleValid);

  operativeUnit_1 : entity work.operativeUnit
    port map (
      I_clock          => I_clock,
      I_reset          => I_reset,
      I_inputSample    => I_inputSample,
      I_loadShift      => SC_loadShift,
      I_initAddress    => SC_initAddress,
      I_incrAddress    => SC_incrAddress,
      I_sel            => SC_sel,
      I_initSum        => SC_initSum,
      I_loadSum        => SC_loadSum,
      I_loadOutA          => SC_loadOutA,
      I_loadOutB          => SC_loadOutB,
      I_loadOutBL   => SC_loadOutBaseLine,
      I_loadOutClellan    => SC_loadOutLissage,
      O_processingDoneA => SC_processingDoneA,
      O_processingDoneB => SC_processingDoneB,
      O_processingDoneBL => SC_processingDoneBaseLine,
      O_processingDoneClellan => SC_processingDoneLissage,
      O_total              => O_filteredSample  );

end architecture archi_firUnit;
